from json import loads, dumps
from openpyxl import Workbook, load_workbook
from re import findall
from datetime import datetime, date

from sterra._sterrage_ import DAY_STRF, HOUR_STRF
from sterra.exterra import exman

class exporter:
    def __init__(self, _:object, List:list[dict], file_path:str, Format:str, **kwargs:dict) -> None:
        self._ = _

        self.list = List
        self.dict_keys = list(self.list[0].keys())
        self.Format = Format
        self.file_path = file_path
        self.kwargs = kwargs
    
    def __call__(self) -> str:
        getattr(self,self.Format.upper())()
        return # (self.id if self.is_part else add(**{"path":self.fp}))

    def XLSX(self) -> Workbook:
        excelfile = Workbook()
        exclsheet = excelfile.active

        usn = self.kwargs.get("username")

        for ik, dk in enumerate(self.dict_keys):
            exclsheet.cell(row=1, column=ik+1, value=dk)

        default_infos = [
            {"v":'S T E R R A'},
            {"v":'=HYPERLINK("https://github.com/novitae", "** Made By novitae **")',"style":'Hyperlink'},
            {"v":'Date :'},
            {"v":f"{date.today().strftime(DAY_STRF)}:{datetime.now().strftime(HOUR_STRF)}"}
            ]
        if self.kwargs.get("target"):
            default_infos.append({"v":"Containing :"})
            default_infos.append({"v":self.kwargs.get("target")})
        if usn:
            default_infos.append({"v":'Targeted account :'})
            default_infos.append({"v":f'=HYPERLINK("https://www.instagram.com/{usn}/", "{usn}")',"style":'Hyperlink'})

        for infnum, inf in enumerate(default_infos):
            exclsheet.cell(row=infnum+1, column=len(self.dict_keys)+1, value=inf["v"]).style=(inf["style"] if "style" in list(inf.keys()) else "Normal")

        for ligne, dico in enumerate(self.list):
            for colonne, val in enumerate(list(dico.values())):
                if val or type(val) is bool:
                    exclsheet.cell(row=ligne+2, column=colonne+1, value=val).style=("Hyperlink" if findall(r'http[s]{0,1}:\/\/', str(val)) and " " not in val else "Normal")

        excelfile.save(self.file_path)

    def CSV(self) -> open:
        with open(self.file_path,"w") as w:
            w.write(','.join(self.dict_keys)+'\n'+"\n".join([",".join([str(v).replace('\n','</b>').replace(',','</c>') for v in d.values()]) for d in self.list]))

    def JSON(self) -> open:
        open(self.file_path,"w").write(dumps(self.list,indent=4))

class reader:
    def __init__(self, _:object, file_path:str) -> None:
        self._ = _
        self.file_path = file_path
        self.f:str = exman(_)._decompose_path(self.file_path)["format"]

    def __call__(self) -> list:
        return getattr(self,self.f.upper())()

    def XLSX(self) -> list:
        excelfile = load_workbook(self.file_path)
        exclsheet = excelfile.active

        maxRow = exclsheet.max_row #! MaxRow prends la longueur de la présentation; si la liste ne contient les données d'un seul username, le maxrow sera toujours de 9
        maxCol = exclsheet.max_column

        rtr = []
        for r in range(2,maxRow):
            dico = {}
            for c in range(1, maxCol):
                val = exclsheet.cell(row=r, column=c).value
                val = val if type(val) != str else val.replace('=HYPERLINK(', '').replace(')', '').split(', ')[0].replace('"', '')
                # Check le type inutile mais on sait jamais si certains chargent des fichiers avec des igid en int
                    
                dico[exclsheet.cell(row=1, column=c).value] = val
            
            if list(dico.values()) != [None]*len(dico): #! Pour éviter l'erreur de maxRow ligne 81
                rtr.append(dico)
        return rtr

    def CSV(self) -> list:
        with open(self.file_path,"r") as r:
            lines = r.read().split("\n")
            keys = lines[0].split(",")
            rtr = [{keys[n]:v.replace('</b>','\n').replace('</c>',',') for n, v in enumerate(preDict.split(","))} for preDict in lines[1:]]
        return rtr

    def JSON(self) -> list:
        return loads(open(self.file_path,"r").read())

# Remplir la case "containg" quand on fait une conversion


from stringcolor import *
import re

LOGOS = {
    "Analyse": cs('|', 'LightSlateBlue')+bold('%').cs('White', 'LightSlateBlue')+cs('|', 'LightSlateBlue'),
    "Compare": cs('|', 'Salmon')+bold('≈').cs('White', 'Salmon')+cs('|', 'Salmon'),
    "csv": cs('|', 'Orange')+bold('c').cs('White', 'Orange')+cs('|', 'Orange'),
    "xlsx": cs('|', 'Green4')+cs('X', 'White', 'Green4')+cs('|', 'Green4'),
    "Exception": cs('|', 'Red2')+bold('!').cs('White', 'Red2')+cs('|', 'Red2'),
    "i": cs('|', 'DodgerBlue2')+bold('i').cs('White', 'DodgerBlue2')+cs('|', 'DodgerBlue2'),
    "Interrogation": cs('|', 'Violet')+bold('?').cs('White', 'Violet')+cs('|', 'Violet'),
    "Instagram": cs('|', 'LightSalmon2')+cs('o', 'White', 'DeepPink4')+cs('|', 'MediumOrchid'),
    "json": cs('{', 'Purple5').bold()+cs('j', 'DeepSkyBlue4').bold()+cs('}', 'Purple5').bold(),
    "Plus": cs('|', 'DarkSeaGreen9')+bold('+').cs('DarkSeaGreen9', 'White')+cs('|', 'DarkSeaGreen9'),
    "Python": cs('|', 'DeepSkyBlue5')+bold('§').cs('LightGoldenrod2', 'DeepSkyBlue5')+cs('|', 'LightGoldenrod2'),
    "Requests": cs('|', 'SeaGreen2')+bold('$').cs('White', 'DarkGrey')+cs('|', 'LightSalmon'),
    "x": cs('|', 'Red2')+bold('x').cs('Red2')+cs('|', 'Red2'),
}

def _no_colors(m:str) -> str:
    # https://stackoverflow.com/a/38662876
    return re.compile(r'(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]').sub('',m)

def _choose_logo(name:str) -> str:
    return LOGOS[name]

class _:
    def __init__(self,colors:bool=True,_raise:bool=False) -> None:        
        self.colors = colors
        self._raise = _raise

    def r(self,e:Exception,cont:bool=False,logo="Exception") -> exit:
        """Raiser"""
        if self._raise:
            raise e
        self.p(f"""{e.__class__.__name__}: {e}""",logo=logo)
        if not cont:
            exit()

    def p(self,message:str,logo:str=None,ext:bool=False) -> print:
        """Printer"""
        message = f"{_choose_logo(logo)} {message}" if logo else message
        print(message if self.colors else _no_colors(message))
        if ext:
            exit()


class _compare:
    def __init__(self,*args) -> None:
        self._, self.l1, self.l2 = args
        try:
            self.idl = [i["id"] for i in self.l1]
        except KeyError:
            self._.r(KeyError("The input list must contain (at least) the key \"id\""))

    def _not_twice(self,l:list) -> list:
        r = []
        for i in l:
            if i not in r:
                r.append(i)
        return r

    def common_usernames(self) -> list:
        """Return list of not common dicts between two list of dicts"""
        return self._not_twice([d for d in self.l2 if d["id"] in self.idl])
    
    def not_common_usernames(self) -> list:
        """Return list of not common dicts between two list of dicts"""
        return self._not_twice([d for d in self.l2 if d["id"] not in self.idl])

    def detail_common(self) -> list:
        """Returns list of dict"""
    #+ faire un detail_common;
    #  il retourne non seulement un dict des commons mais aussi un dict des commons qui ont des différences entre la première et la seconde liste 


from setuptools import setup, find_packages

setup(
    name = 'sterra',
    version = '2.2.2',
    description = 'A SOCMINT tool to analyse instagram followers | following of a target, and get informations from it.',
    author = 'novitae',
    url = 'https://github.com/novitae/sterraxcyl',
    license = 'GNU General Public License v3 (GPLv3)',
    classifiers = [
        'Programming Language :: Python :: 3.9',
    ],
    packages = find_packages(),
    install_requires = ['aiohttp', 'argparse', 'datetime', 'openpyxl', 'requests', 'string-color', 'tqdm'],
    entry_points = {'console_scripts': ['sterra = sterra.core:main']}
)



USER_AGENTS = {
    "safari": [
        "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
        "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/534.15+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
        "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
        "Mozilla/5.0 (Android 2.2; Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
        "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-HK) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 6.0; tr-TR) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 6.0; nb-NO) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-TW) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
        "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5"
        ],
    "ios": [
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone12,8; iOS 14_8; en_CL; en-CL; scale=2.00; 750x1334; 327311214)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone11,8; iOS 15_0_1; es_CL; es-CL; scale=2.00; 828x1792; 325544617)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,1; iOS 14_7_1; en_SA@calendar=gregorian; en-SA; scale=2.88; 1080x2338; 329643252) NW/3",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,1; iOS 14_7_1; en_SA@calendar=gregorian; en-SA; scale=2.88; 1080x2338; 329643252)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone14,2; iOS 15_1; da_DK; da-DK; scale=3.00; 1170x2532; 330663239)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8; pt_BR; pt-BR; scale=2.00; 828x1792; 330663239)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,3; iOS 14_7_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 329643252) NW/1",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,4; iOS 15_1; it_IT; it; scale=3.00; 1284x2778; 330663239) NW/3",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,1; iOS 15_0_2; ru_RU; ru; scale=2.88; 1080x2338; 330663239)",
        "Mozilla/5.0 (iPad; CPU OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPad8,9; iPadOS 15_1; en_IT; en-GB; scale=2.00; 750x1334; 330663239)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone12,3; iOS 13_5_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 322184766) NW/3",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone12,3; iOS 13_5_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 322184766)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8_1; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239) NW/3",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8_1; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239)",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,2; iOS 15_0; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239) NW/1",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone9,1; iOS 14_8_1; it_IT; it-IT; scale=2.00; 750x1334; 330663239)",
        ]
    }

